@@ -63,44 +63,50 @@ static bool connected = true;
 static char *username = NULL;
 static char *password = NULL;
 static bool disconnect_sent = false;
+static bool quiet = false;
 
 void my_connect_callback(void *obj, int result)
 {
 	struct mosquitto *mosq = obj;
+	int rc = MOSQ_ERR_SUCCESS;
 
 	if(!result){
 		switch(mode){
 			case MSGMODE_CMD:
 			case MSGMODE_FILE:
 			case MSGMODE_STDIN_FILE:
-				mosquitto_publish(mosq, &mid_sent, topic, msglen, (uint8_t *)message, qos, retain);
+				rc = mosquitto_publish(mosq, &mid_sent, topic, msglen, (uint8_t *)message, qos, retain);
 				break;
 			case MSGMODE_NULL:
-				mosquitto_publish(mosq, &mid_sent, topic, 0, NULL, qos, retain);
+				rc = mosquitto_publish(mosq, &mid_sent, topic, 0, NULL, qos, retain);
 				break;
 			case MSGMODE_STDIN_LINE:
 				status = STATUS_CONNACK_RECVD;
 				break;
 		}
+		if(rc){
+			if(!quiet) fprintf(stderr, "Error: Publish returned %d, disconnecting.\n", rc);
+			mosquitto_disconnect(mosq);
+		}
 	}else{
 		switch(result){
 			case 1:
-				fprintf(stderr, "Connection Refused: unacceptable protocol version\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: unacceptable protocol version\n");
 				break;
 			case 2:
-				fprintf(stderr, "Connection Refused: identifier rejected\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: identifier rejected\n");
 				break;
 			case 3:
-				fprintf(stderr, "Connection Refused: broker unavailable\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: broker unavailable\n");
 				break;
 			case 4:
-				fprintf(stderr, "Connection Refused: bad user name or password\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: bad user name or password\n");
 				break;
 			case 5:
-				fprintf(stderr, "Connection Refused: not authorised\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: not authorised\n");
 				break;
 			default:
-				fprintf(stderr, "Connection Refused: unknown reason\n");
+				if(!quiet) fprintf(stderr, "Connection Refused: unknown reason\n");
 				break;
 		}
 	}
@@ -132,7 +138,7 @@ int load_stdin(void)
 		rlen = fread(buf, 1, 1024, stdin);
 		message = realloc(message, pos+rlen);
 		if(!message){
-			fprintf(stderr, "Error: Out of memory.\n");
+			if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
 			return 1;
 		}
 		memcpy(&(message[pos]), buf, rlen);
@@ -141,7 +147,7 @@ int load_stdin(void)
 	msglen = pos;
 
 	if(!msglen){
-		fprintf(stderr, "Error: Zero length input.\n");
+		if(!quiet) fprintf(stderr, "Error: Zero length input.\n");
 		return 1;
 	}
 
@@ -155,7 +161,7 @@ int load_file(const char *filename)
 
 	fptr = fopen(filename, "rb");
 	if(!fptr){
-		fprintf(stderr, "Error: Unable to open file \"%s\".\n", filename);
+		if(!quiet) fprintf(stderr, "Error: Unable to open file \"%s\".\n", filename);
 		return 1;
 	}
 	mode = MSGMODE_FILE;
@@ -163,19 +169,19 @@ int load_file(const char *filename)
 	msglen = ftell(fptr);
 	if(msglen > 268435455){
 		fclose(fptr);
-		fprintf(stderr, "Error: File \"%s\" is too large (>268,435,455 bytes).\n", filename);
+		if(!quiet) fprintf(stderr, "Error: File \"%s\" is too large (>268,435,455 bytes).\n", filename);
 		return 1;
 	}
 	if(msglen == 0){
 		fclose(fptr);
-		fprintf(stderr, "Error: File \"%s\" is empty.\n", filename);
+		if(!quiet) fprintf(stderr, "Error: File \"%s\" is empty.\n", filename);
 		return 1;
 	}
 	fseek(fptr, 0, SEEK_SET);
 	message = malloc(msglen);
 	if(!message){
 		fclose(fptr);
-		fprintf(stderr, "Error: Out of memory.\n");
+		if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
 		return 1;
 	}
 	pos = 0;
@@ -190,13 +196,17 @@ int load_file(const char *filename)
 void print_usage(void)
 {
 	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n\n");
-	printf("Usage: mosquitto_pub [-d] [-h host] [-i id] [-p port] [-q qos] [-r] {-f file | -l | -n | -m message} -t topic\n");
-	printf("                     [-u username [--pw password]]\n");
+	printf("Usage: mosquitto_pub [-h host] [-p port] [-q qos] [-r] {-f file | -l | -n | -m message} -t topic\n");
+	printf("                     [-i id] [-I id_prefix]\n");
+	printf("                     [-d] [--quiet]\n");
+	printf("                     [-u username [-P password]]\n");
 	printf("                     [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n\n");
 	printf(" -d : enable debug messages.\n");
 	printf(" -f : send the contents of a file as the message.\n");
 	printf(" -h : mqtt host to connect to. Defaults to localhost.\n");
 	printf(" -i : id to use for this client. Defaults to mosquitto_pub_ appended with the process id.\n");
+	printf(" -I : define the client id as id_prefix appended with the process id. Useful for when the\n");
+	printf("      broker is using the clientid_prefixes option.\n");
 	printf(" -l : read messages from stdin, sending a separate message for each line.\n");
 	printf(" -m : message payload to send.\n");
 	printf(" -n : send a null (zero length) message.\n");
@@ -206,7 +216,8 @@ void print_usage(void)
 	printf(" -s : read message from stdin, sending the entire input as a message.\n");
 	printf(" -t : mqtt topic to publish to.\n");
 	printf(" -u : provide a username (requires MQTT 3.1 broker)\n");
-	printf(" --pw : provide a password (requires MQTT 3.1 broker)\n");
+	printf(" -P : provide a password (requires MQTT 3.1 broker)\n");
+	printf(" --quiet : don't print error messages.\n");
 	printf(" --will-payload : payload for the client Will, which is sent by the broker in case of\n");
 	printf("                  unexpected disconnection. If not given and will-topic is set, a zero\n");
 	printf("                  length message will be sent.\n");
@@ -218,7 +229,8 @@ void print_usage(void)
 
 int main(int argc, char *argv[])
 {
-	char id[30];
+	char *id = NULL;
+	char *id_prefix = NULL;
 	int i;
 	char *host = "localhost";
 	int port = 1883;
@@ -227,6 +239,8 @@ int main(int argc, char *argv[])
 	char buf[1024];
 	bool debug = false;
 	struct mosquitto *mosq = NULL;
+	int rc;
+	int rc2;
 
 	uint8_t *will_payload = NULL;
 	long will_payloadlen = 0;
@@ -234,8 +248,6 @@ int main(int argc, char *argv[])
 	bool will_retain = false;
 	char *will_topic = NULL;
 
-	snprintf(id, 30, "mosquitto_pub_%d", getpid());
-
 	for(i=1; i<argc; i++){
 		if(!strcmp(argv[i], "-p") || !strcmp(argv[i], "--port")){
 			if(i==argc-1){
@@ -276,13 +288,31 @@ int main(int argc, char *argv[])
 			}
 			i++;
 		}else if(!strcmp(argv[i], "-i") || !strcmp(argv[i], "--id")){
+			if(id_prefix){
+				fprintf(stderr, "Error: -i and -I argument cannot be used together.\n\n");
+				print_usage();
+				return 1;
+			}
 			if(i==argc-1){
 				fprintf(stderr, "Error: -i argument given but no id specified.\n\n");
 				print_usage();
 				return 1;
 			}else{
-				memset(id, 0, 30);
-				snprintf(id, 29, "%s", argv[i+1]);
+				id = argv[i+1];
+			}
+			i++;
+		}else if(!strcmp(argv[i], "-I") || !strcmp(argv[i], "--id-prefix")){
+			if(id){
+				fprintf(stderr, "Error: -i and -I argument cannot be used together.\n\n");
+				print_usage();
+				return 1;
+			}
+			if(i==argc-1){
+				fprintf(stderr, "Error: -I argument given but no id prefix specified.\n\n");
+				print_usage();
+				return 1;
+			}else{
+				id_prefix = argv[i+1];
 			}
 			i++;
 		}else if(!strcmp(argv[i], "-l") || !strcmp(argv[i], "--stdin-line")){
@@ -337,6 +367,8 @@ int main(int argc, char *argv[])
 				}
 			}
 			i++;
+		}else if(!strcmp(argv[i], "--quiet")){
+			quiet = true;
 		}else if(!strcmp(argv[i], "-r") || !strcmp(argv[i], "--retain")){
 			retain = 1;
 		}else if(!strcmp(argv[i], "-s") || !strcmp(argv[i], "--stdin-file")){
@@ -365,9 +397,9 @@ int main(int argc, char *argv[])
 				username = argv[i+1];
 			}
 			i++;
-		}else if(!strcmp(argv[i], "--pw")){
+		}else if(!strcmp(argv[i], "-P") || !strcmp(argv[i], "--pw")){
 			if(i==argc-1){
-				fprintf(stderr, "Error: --pw argument given but no password specified.\n\n");
+				fprintf(stderr, "Error: -P argument given but no password specified.\n\n");
 				print_usage();
 				return 1;
 			}else{
@@ -414,6 +446,22 @@ int main(int argc, char *argv[])
 			return 1;
 		}
 	}
+	if(id_prefix){
+		id = malloc(strlen(id_prefix)+10);
+		if(!id){
+			if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
+			return 1;
+		}
+		snprintf(id, strlen(id_prefix)+10, "%s%d", id_prefix, getpid());
+	}else if(!id){
+		id = malloc(30);
+		if(!id){
+			if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
+			return 1;
+		}
+		snprintf(id, 30, "mosquitto_pub_%d", getpid());
+	}
+
 	if(!topic || mode == MSGMODE_NONE){
 		fprintf(stderr, "Error: Both topic and message must be supplied.\n");
 		print_usage();
@@ -431,12 +479,12 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 	if(password && !username){
-		fprintf(stderr, "Warning: Not using password since username not set.\n");
+		if(!quiet) fprintf(stderr, "Warning: Not using password since username not set.\n");
 	}
 	mosquitto_lib_init();
 	mosq = mosquitto_new(id, NULL);
 	if(!mosq){
-		fprintf(stderr, "Error: Out of memory.\n");
+		if(!quiet) fprintf(stderr, "Error: Out of memory.\n");
 		return 1;
 	}
 	if(debug){
@@ -444,11 +492,11 @@ int main(int argc, char *argv[])
 				| MOSQ_LOG_NOTICE | MOSQ_LOG_INFO, MOSQ_LOG_STDERR);
 	}
 	if(will_topic && mosquitto_will_set(mosq, true, will_topic, will_payloadlen, will_payload, will_qos, will_retain)){
-		fprintf(stderr, "Error: Problem setting will.\n");
+		if(!quiet) fprintf(stderr, "Error: Problem setting will.\n");
 		return 1;
 	}
 	if(username && mosquitto_username_pw_set(mosq, username, password)){
-		fprintf(stderr, "Error: Problem setting username and password.\n");
+		if(!quiet) fprintf(stderr, "Error: Problem setting username and password.\n");
 		return 1;
 	}
 
@@ -456,26 +504,33 @@ int main(int argc, char *argv[])
 	mosquitto_disconnect_callback_set(mosq, my_disconnect_callback);
 	mosquitto_publish_callback_set(mosq, my_publish_callback);
 
-	if(mosquitto_connect(mosq, host, port, keepalive, true)){
-		fprintf(stderr, "Unable to connect.\n");
-		return 1;
+	rc = mosquitto_connect(mosq, host, port, keepalive, true);
+	if(rc){
+		if(!quiet) fprintf(stderr, "Unable to connect (%d).\n", rc);
+		return rc;
 	}
 
-	while(!mosquitto_loop(mosq, -1) && connected){
+	do{
 		if(mode == MSGMODE_STDIN_LINE && status == STATUS_CONNACK_RECVD){
 			if(fgets(buf, 1024, stdin)){
 				buf[strlen(buf)-1] = '\0';
-				mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), (uint8_t *)buf, qos, retain);
+				rc2 = mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), (uint8_t *)buf, qos, retain);
+				if(rc2){
+					if(!quiet) fprintf(stderr, "Error: Publish returned %d, disconnecting.\n", rc2);
+					mosquitto_disconnect(mosq);
+				}
 			}else if(feof(stdin) && disconnect_sent == false){
 				mosquitto_disconnect(mosq);
 				disconnect_sent = true;
 			}
 		}
-	}
+		rc = mosquitto_loop(mosq, -1);
+	}while(rc == MOSQ_ERR_SUCCESS && connected);
+
 	if(message && mode == MSGMODE_FILE){
 		free(message);
 	}
 	mosquitto_destroy(mosq);
 	mosquitto_lib_cleanup();
-	return 0;
+	return rc;
 }
