@@ -115,6 +115,8 @@ struct mosquitto *mosquitto_new(const char *id, void *obj)
 		mosq->on_unsubscribe = NULL;
 		mosq->log_destinations = MOSQ_LOG_NONE;
 		mosq->log_priorities = MOSQ_LOG_ERR | MOSQ_LOG_WARNING | MOSQ_LOG_NOTICE | MOSQ_LOG_INFO;
+		mosq->host = NULL;
+		mosq->port = 1883;
 #ifdef WITH_SSL
 		mosq->core.ssl = NULL;
 #endif
@@ -221,8 +223,11 @@ void mosquitto_destroy(struct mosquitto *mosq)
 	if(mosq->core.will){
 		if(mosq->core.will->topic) _mosquitto_free(mosq->core.will->topic);
 		if(mosq->core.will->payload) _mosquitto_free(mosq->core.will->payload);
+		_mosquitto_free(mosq->core.will);
+	}
+	if(mosq->host){
+		_mosquitto_free(mosq->host);
 	}
-	_mosquitto_free(mosq->core.will);
 #ifdef WITH_SSL
 	if(mosq->core.ssl){
 		if(mosq->core.ssl->ssl){
@@ -244,17 +249,33 @@ int mosquitto_socket(struct mosquitto *mosq)
 }
 
 int mosquitto_connect(struct mosquitto *mosq, const char *host, int port, int keepalive, bool clean_session)
+{
+	if(!mosq) return MOSQ_ERR_INVAL;
+	if(!host || port <= 0) return MOSQ_ERR_INVAL;
+
+	if(mosq->host) _mosquitto_free(mosq->host);
+	mosq->host = _mosquitto_strdup(host);
+	if(!mosq->host) return MOSQ_ERR_NOMEM;
+	mosq->port = port;
+
+	mosq->core.keepalive = keepalive;
+	mosq->core.clean_session = clean_session;
+
+	return mosquitto_reconnect(mosq);
+}
+
+int mosquitto_reconnect(struct mosquitto *mosq)
 {
 	int rc;
 	if(!mosq) return MOSQ_ERR_INVAL;
-	if(!host || !port) return MOSQ_ERR_INVAL;
+	if(!mosq->host || mosq->port <= 0) return MOSQ_ERR_INVAL;
 
-	rc = _mosquitto_socket_connect(&mosq->core, host, port);
+	rc = _mosquitto_socket_connect(&mosq->core, mosq->host, mosq->port);
 	if(rc){
 		return rc;
 	}
 
-	return _mosquitto_send_connect(mosq, keepalive, clean_session);
+	return _mosquitto_send_connect(&mosq->core, mosq->core.keepalive, mosq->core.clean_session);
 }
 
 int mosquitto_disconnect(struct mosquitto *mosq)
@@ -264,7 +285,7 @@ int mosquitto_disconnect(struct mosquitto *mosq)
 
 	mosq->core.state = mosq_cs_disconnecting;
 
-	return _mosquitto_send_disconnect(mosq);
+	return _mosquitto_send_disconnect(&mosq->core);
 }
 
 int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic, uint32_t payloadlen, const uint8_t *payload, int qos, bool retain)
@@ -275,6 +296,10 @@ int mosquitto_publish(struct mosquitto *mosq, uint16_t *mid, const char *topic,
 	if(!mosq || !topic || qos<0 || qos>2) return MOSQ_ERR_INVAL;
 	if(payloadlen > 268435455) return MOSQ_ERR_PAYLOAD_SIZE;
 
+	if(_mosquitto_wildcard_check(topic)){
+		return MOSQ_ERR_INVAL;
+	}
+
 	local_mid = _mosquitto_mid_generate(&mosq->core);
 	if(mid){
 		*mid = local_mid;
@@ -326,7 +351,7 @@ int mosquitto_subscribe(struct mosquitto *mosq, uint16_t *mid, const char *sub,
 	if(!mosq) return MOSQ_ERR_INVAL;
 	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
-	return _mosquitto_send_subscribe(mosq, mid, false, sub, qos);
+	return _mosquitto_send_subscribe(&mosq->core, mid, false, sub, qos);
 }
 
 int mosquitto_unsubscribe(struct mosquitto *mosq, uint16_t *mid, const char *sub)
@@ -334,7 +359,7 @@ int mosquitto_unsubscribe(struct mosquitto *mosq, uint16_t *mid, const char *sub
 	if(!mosq) return MOSQ_ERR_INVAL;
 	if(mosq->core.sock == INVALID_SOCKET) return MOSQ_ERR_NO_CONN;
 
-	return _mosquitto_send_unsubscribe(mosq, mid, false, sub);
+	return _mosquitto_send_unsubscribe(&mosq->core, mid, false, sub);
 }
 
 #if 0
@@ -411,13 +436,12 @@ int mosquitto_loop(struct mosquitto *mosq, int timeout)
 			if(rc){
 				_mosquitto_socket_close(&mosq->core);
 				if(mosq->core.state == mosq_cs_disconnecting){
-					if(mosq->on_disconnect){
-						mosq->on_disconnect(mosq->obj);
-					}
-					return MOSQ_ERR_SUCCESS;
-				}else{
-					return rc;
+					rc = MOSQ_ERR_SUCCESS;
 				}
+				if(mosq->on_disconnect){
+					mosq->on_disconnect(mosq->obj);
+				}
+				return rc;
 			}
 		}
 		if(FD_ISSET(mosq->core.sock, &writefds)){
@@ -425,13 +449,12 @@ int mosquitto_loop(struct mosquitto *mosq, int timeout)
 			if(rc){
 				_mosquitto_socket_close(&mosq->core);
 				if(mosq->core.state == mosq_cs_disconnecting){
-					if(mosq->on_disconnect){
-						mosq->on_disconnect(mosq->obj);
-					}
-					return MOSQ_ERR_SUCCESS;
-				}else{
-					return rc;
+					rc = MOSQ_ERR_SUCCESS;
 				}
+				if(mosq->on_disconnect){
+					mosq->on_disconnect(mosq->obj);
+				}
+				return rc;
 			}
 		}
 	}
