@@ -135,6 +135,8 @@ struct mosquitto {
 	void (*on_subscribe)(void *obj, uint16_t mid, int qos_count, const uint8_t *granted_qos);
 	void (*on_unsubscribe)(void *obj, uint16_t mid);
 	//void (*on_error)();
+	char *host;
+	int port;
 };
 
 #endif
