@@ -138,6 +138,7 @@ class Mosquitto:
 		else:
 			self.obj = obj
 
+		_mosquitto_lib_init()
 		self._mosq = _mosquitto_new(id, None)
 
 		#==================================================
@@ -145,11 +146,11 @@ class Mosquitto:
 		#==================================================
 		self._internal_on_connect_cast = _MOSQ_CONNECT_FUNC(self._internal_on_connect)
 		_mosquitto_connect_callback_set(self._mosq, self._internal_on_connect_cast)
-		self._on_connect = None
+		self.on_connect = None
 	
 		self._internal_on_disconnect_cast = _MOSQ_DISCONNECT_FUNC(self._internal_on_disconnect)
 		_mosquitto_disconnect_callback_set(self._mosq, self._internal_on_disconnect_cast)
-		self._on_disconnect = None
+		self.on_disconnect = None
 	
 		self._internal_on_message_cast = _MOSQ_MESSAGE_FUNC(self._internal_on_message)
 		_mosquitto_message_callback_set(self._mosq, self._internal_on_message_cast)
@@ -170,6 +171,9 @@ class Mosquitto:
 		# End configure callbacks
 		#==================================================
 
+	def __del__(self):
+		_mosquitto_lib_cleanup()
+		
 	def __del__(self):
 		if self._mosq:
 			_mosquitto_destroy(self._mosq)
@@ -197,6 +201,11 @@ class Mosquitto:
 		"""
 		return _mosquitto_connect(self._mosq, hostname, port, keepalive, clean_session)
 
+	def reconnect(self):
+		"""Reconnect to a broker. This uses the saved parameters from the
+		connect() call to reconnect to a broker after a disconnect."""
+		return _mosquitto_reconnect(self._mosq)
+
 	def disconnect(self):
 		"""Disconnect a connected client from the broker."""
 		return _mosquitto_disconnect(self._mosq)
@@ -327,7 +336,7 @@ class Mosquitto:
 
 			if argcount == 1:
 				self.on_connect(rc)
-			elif argcount == 2:
+			elif argcount == 2 or argcount == 3:
 				self.on_connect(self.obj, rc)
 
 	def _internal_on_disconnect(self, obj):
@@ -336,21 +345,22 @@ class Mosquitto:
 
 			if argcount == 0:
 				self.on_disconnect()
-			elif argcount == 1:
+			elif argcount == 1 or argcount == 2:
 				self.on_disconnect(self.obj)
 
 	def _internal_on_message(self, obj, message):
 		if self.on_message:
 			topic = message.contents.topic
+			payloadlen = message.contents.payloadlen
 			payload = message.contents.payload
 			qos = message.contents.qos
 			retain = message.contents.retain
-			msg = MosquittoMessage(topic, payload, qos, retain)
+			msg = MosquittoMessage(topic, payloadlen, payload, qos, retain)
 			argcount = self.on_message.func_code.co_argcount
 
 			if argcount == 1:
 				self.on_message(msg)
-			elif argcount == 2:
+			elif argcount == 2 or argcount == 3:
 				self.on_message(self.obj, msg)
 
 	def _internal_on_publish(self, obj, mid):
@@ -359,7 +369,7 @@ class Mosquitto:
 
 			if argcount == 1:
 				self.on_publish(mid)
-			elif argcount == 2:
+			elif argcount == 2 or argcount == 3:
 				self.on_publish(self.obj, mid)
 
 	def _internal_on_subscribe(self, obj, mid, qos_count, granted_qos):
@@ -371,7 +381,7 @@ class Mosquitto:
 
 			if argcount == 2:
 				self.on_subscribe(mid, qos_list)
-			elif argcount == 3:
+			elif argcount == 3 or argcount == 4:
 				self.on_subscribe(self.obj, mid, qos_list)
 
 	def _internal_on_unsubscribe(self, obj, mid):
@@ -380,7 +390,7 @@ class Mosquitto:
 
 			if argcount == 1:
 				self.on_unsubscribe(mid)
-			elif argcount == 2:
+			elif argcount == 2 or argcount == 3:
 				self.on_unsubscribe(self.obj, mid)
 
 class c_MosquittoMessage(Structure):
@@ -389,16 +399,18 @@ class c_MosquittoMessage(Structure):
 	Don't use."""
 	_fields_ = [("mid", c_uint16),
 				("topic", c_char_p),
-				("payload", c_char_p),
+				("payload", POINTER(c_uint8)),
 				("payloadlen", c_uint32),
 				("qos", c_int),
 				("retain", c_bool)]
 
 class MosquittoMessage:
 	"""MQTT message class"""
-	def __init__(self, topic, payload, qos, retain):
+	def __init__(self, topic, payloadlen, payload, qos, retain):
 		self.topic = topic
+		self.payloadlen = payloadlen
 		self.payload = payload
+		self.payload_str = cast(self.payload, c_char_p).value
 		self.qos = qos
 		self.retain = retain
 
@@ -406,6 +418,15 @@ class MosquittoMessage:
 # Library loading
 #==================================================
 _libmosq = cdll.LoadLibrary(find_library("mosquitto"))
+
+_mosquitto_lib_init = _libmosq.mosquitto_lib_init
+_mosquitto_lib_init.argtypes = None
+_mosquitto_lib_init.restype = c_int
+
+_mosquitto_lib_cleanup = _libmosq.mosquitto_lib_cleanup
+_mosquitto_lib_cleanup.argtypes = None
+_mosquitto_lib_cleanup.restype = c_int
+
 _mosquitto_new = _libmosq.mosquitto_new
 _mosquitto_new.argtypes = [c_char_p, c_void_p]
 _mosquitto_new.restype = c_void_p
@@ -418,6 +439,10 @@ _mosquitto_connect = _libmosq.mosquitto_connect
 _mosquitto_connect.argtypes = [c_void_p, c_char_p, c_int, c_int, c_bool]
 _mosquitto_connect.restype = c_int
 
+_mosquitto_reconnect = _libmosq.mosquitto_reconnect
+_mosquitto_reconnect.argtypes = [c_void_p]
+_mosquitto_reconnect.restype = c_int
+
 _mosquitto_disconnect = _libmosq.mosquitto_disconnect
 _mosquitto_disconnect.argtypes = [c_void_p]
 _mosquitto_disconnect.restype = c_int
