@@ -1,5 +1,6 @@
 /*
 Copyright (c) 2010,2011 Roger Light <roger@atchoo.org>
+Copyright (c) 2011 Sang Kyeong Nam
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -70,7 +71,7 @@ static int _subs_process(struct _mosquitto_db *db, struct _mosquitto_subhier *hi
 			continue;
 		}
 		/* Check for ACL topic access. */
-		rc2 = mqtt3_acl_check(db, leaf->context, topic, MOSQ_ACL_READ);
+		rc2 = mosquitto_acl_check(db, leaf->context, topic, MOSQ_ACL_READ);
 		if(rc2 == MOSQ_ERR_ACL_DENIED){
 			leaf = leaf->next;
 			continue;
@@ -525,7 +526,7 @@ static int _retain_process(struct _mosquitto_db *db, struct mosquitto_msg_store
 	int qos;
 	uint16_t mid;
 
-	rc = mqtt3_acl_check(db, context, retained->msg.topic, MOSQ_ACL_READ);
+	rc = mosquitto_acl_check(db, context, retained->msg.topic, MOSQ_ACL_READ);
 	if(rc == MOSQ_ERR_ACL_DENIED){
 		return MOSQ_ERR_SUCCESS;
 	}else if(rc != MOSQ_ERR_SUCCESS){
@@ -552,13 +553,13 @@ static int _retain_search(struct _mosquitto_db *db, struct _mosquitto_subhier *s
 		/* Subscriptions with wildcards in aren't really valid topics to publish to
 		 * so they can't have retained messages.
 		 */
-		if(strcmp(branch->topic, "+") && strcmp(branch->topic, "#")){
+		if(!_mosquitto_wildcard_check(branch->topic)){
 			if(!strcmp(tokens->topic, "#") && !tokens->next){
 				if(branch->retained){
 					_retain_process(db, branch->retained, context, sub, sub_qos);
 				}
 				_retain_search(db, branch, tokens, context, sub, sub_qos);
-			}else if(!strcmp(branch->topic, tokens->topic) || !strcmp(branch->topic, "+")){
+			}else if(!strcmp(branch->topic, tokens->topic) || !strcmp(tokens->topic, "+")){
 				if(tokens->next){
 					_retain_search(db, branch, tokens->next, context, sub, sub_qos);
 				}else{
